package com.sooka.utils;

import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;
import java.security.SecureRandom;

/**
 * 秘钥生成器工具类
 *
 * @author lei_wang
 */
public class SecretKeyUtil {


    /**
     * 加密
     * 1.构造**生成器
     * 2.根据ecnodeRules规则初始化**生成器
     * 3.产生**
     * 4.创建和初始化密码器
     * 5.内容加密
     * 6.返回字符串
     */
    public static String AESEncode(String encodeRules, String content) {
        try {
            //1.构造**生成器，指定为AES算法,不区分大小写
            KeyGenerator aes = KeyGenerator.getInstance("AES");
            //2.根据ecnodeRules规则初始化**生成器
            //生成一个128位的随机源,根据传入的字节数组
//            aes.init(128, new SecureRandom(encodeRules.getBytes()));
            SecureRandom secureRandom = SecureRandom.getInstance("SHA1PRNG");
            secureRandom.setSeed(encodeRules.getBytes());
            aes.init(128, secureRandom);
            //3.产生原始对称**
            SecretKey original_key = aes.generateKey();
            //4.获得原始对称**的字节数组
            byte[] raw = original_key.getEncoded();
            //5.根据字节数组生成AES**
            SecretKey key = new SecretKeySpec(raw, "AES");
            //6.根据指定算法AES自成密码器
            Cipher cipher = Cipher.getInstance("AES");
            //7.初始化密码器，第一个参数为加密(Encrypt_mode)或者解密解密(Decrypt_mode)操作，第二个参数为使用的KEY
            cipher.init(Cipher.ENCRYPT_MODE, key);
            //8.获取加密内容的字节数组(这里要设置为utf-8)不然内容中如果有中文和英文混合中文就会解密为乱码
            byte[] byte_encode = content.getBytes("utf-8");
            //9.根据密码器的初始化方式--加密：将数据加密
            byte[] byte_AES = cipher.doFinal(byte_encode);
            //10.将加密后的数据转换为字符串
            //这里用Base64Encoder中会找不到包
            //解决办法：
            //在项目的Build path中先移除JRE System Library，再添加库JRE System Library，重新编译后就一切正常了。
            String AES_encode = new String(new BASE64Encoder().encode(byte_AES));
            //11.将字符串返回
            return AES_encode;
        } catch (Exception e) {
            e.printStackTrace();
        }
        //如果有错就返加nulll
        return null;
    }

    /**
     * 解密
     * 解密过程：
     * 1.同加密1-4步
     * 2.将加密后的字符串反纺成byte[]数组
     * 3.将加密内容解密
     */
    public static String AESDncode(String encodeRules, String content) {
        try {
            //1.构造**生成器，指定为AES算法,不区分大小写
            KeyGenerator keyGenerator = KeyGenerator.getInstance("AES");
            //2.根据ecnodeRules规则初始化**生成器
            //生成一个128位的随机源,根据传入的字节数组
//            keyGenerator.init(128, new SecureRandom(encodeRules.getBytes()));
            SecureRandom secureRandom = SecureRandom.getInstance("SHA1PRNG");
            secureRandom.setSeed(encodeRules.getBytes());
            keyGenerator.init(128, secureRandom);
            //3.产生原始对称**
            SecretKey original_key = keyGenerator.generateKey();
            //4.获得原始对称**的字节数组
            byte[] raw = original_key.getEncoded();
            //5.根据字节数组生成AES**
            SecretKey key = new SecretKeySpec(raw, "AES");
            //6.根据指定算法AES自成密码器
            Cipher cipher = Cipher.getInstance("AES");
            //7.初始化密码器，第一个参数为加密(Encrypt_mode)或者解密(Decrypt_mode)操作，第二个参数为使用的KEY
            cipher.init(Cipher.DECRYPT_MODE, key);
            //8.将加密并编码后的内容解码成字节数组
            byte[] byte_content = new BASE64Decoder().decodeBuffer(content);
            /*
             * 解密
             */
            byte[] byte_decode = cipher.doFinal(byte_content);
            String AES_decode = new String(byte_decode, "utf-8");
            return AES_decode;
        } catch (Exception e) {
            System.out.println(e.getMessage());
            e.printStackTrace();
        }
        //如果有错就返加nulll
        return null;
    }

    public static void main(String[] args) {
        /*
         * 加密
         */
//        String str = SecretKeyUtil.AESEncode("3fc674da58", "123/政数局/2022-10-20 - 2022-10-29/b29c94d72ee74c6d94b573d90020ea59");
        String str = "1HjlUwOjkNCuvi+E8+r+4oYWPV1lDMF6l8E7GovGe9h4GtK4nDBXQKirD57TTku+08MHkHsS04X8xpKGQTrNOxG5LfweCxtHbmoUczwF9Y8=";
        System.out.println("加密后的密文是:" + str);

        /*
         * 解密
         */
        System.out.println("解密后的明文是:" + SecretKeyUtil.AESDncode("3fc674da58", str));

        /*
         * 解密
         */
        System.out.println("解密后的明文是:" + SecretKeyUtil.AESDncode("3fc674da58", "fu0uwO5T2p9Dn3j9TvhpKbC2nCDoDli1TUONB6uZH7rzXgn6vAQK+h/yjYy5iWLY/l/67wygmv825wvKmpjoxTtc3ANl4VhVxhkosEsv5Lc="));
        System.out.println("解密后的明文是:" + SecretKeyUtil.AESDncode("3fc674da58", "1HjlUwOjkNCuvi+E8+r+4oYWPV1lDMF6l8E7GovGe9h4GtK4nDBXQKirD57TTku+08MHkHsS04X8\n" +
                "xpKGQTrNOxG5LfweCxtHbmoUczwF9Y8="));
    }
}
