package com.sooka.model;

import com.google.gson.annotations.Expose;
import com.sooka.enums.ResultEnum;
import lombok.Data;

@Data
public class ResultModel<T> {


    //错误码
    @Expose
    private Integer code;
    //提示信息
    @Expose
    private String message;
    //返回数据
    @Expose
    private T data;


    public static <T> ResultModel<T> ok() {
        return new ResultModel<T>(ResultEnum.OK);
    }

    public static <T> ResultModel<T> ok(T data) {
        return new ResultModel<T>(ResultEnum.OK, data);
    }

    public static <T> ResultModel<T> error(String message) {
        return new ResultModel<T>(ResultEnum.ERROR.getCode(), message);
    }

    public static <T> ResultModel<T> error(ResultEnum code) {
        return new ResultModel<T>(code);
    }

    private ResultModel(ResultEnum code) {
        this.code = code.getCode();
        this.message = code.getMessage();
        this.data = null;
    }

    private ResultModel(ResultEnum code, T data) {
        this.code = code.getCode();
        this.message = code.getMessage();
        this.data = data;
    }

    private ResultModel(Integer code, String message) {
        this.code = code;
        this.message = message;
        this.data = null;
    }

    public ResultModel() {

    }

}
